#pragma rtGlobals=1		// Use modern global access method.
#Include <Remove Points> // This allows use of the remove NaNs function from wavemetrics

//Voir si borne1[counter] et borne2[counter] < peak_wave_ch1[counter]
Function MAS(peak_wave_ch1, Tags_ch1, True_wave_ch2, Amp_wave_ch1, peak_win_width, Bsl_width, AP_rise_time, Lag_to_peak, [lower_lim, upper_lim, compteur]) //peak_wave should be RealPeakPosition of ch1
	Wave peak_wave_ch1, Tags_ch1, True_wave_ch2, Amp_wave_ch1
	Variable peak_win_width, Bsl_width, AP_rise_time, Lag_to_peak // lag_to_peak is mean time interval between peak on ch1 and peak on ch2
	Variable lower_lim, upper_lim, compteur // parameters that should become optionnal
	
	
	//Set range indexing variables for future use
	Variable/G lower_limit_P, upper_limit_P// limits in point indexing ofanalysis on Action Potentials wave
	
	Variable random_window = 100// To be set in ms (for reading convenience)
	random_window *= 0.001//Converted in secs
	
	// finds the point in peak_wave_ch1 closest to lower_limit_P (point of the wave where analysis beggins)
	Lower_limit_P = X_finder(peak_wave_ch1, lower_lim, supBorder=1)
	// Same thing for upper limit
	Upper_limit_P = X_finder(peak_wave_ch1, upper_lim, infBorder=1)
	
	//Output waves
	Duplicate/O/R=[Lower_limit_P, upper_limit_P] peak_wave_ch1, TEMPO_Deltas
	TEMPO_Deltas = NaN //defaults value is NaN (used to remove points later)
	Make/O/N=(3 * numpnts(TEMPO_Deltas)) rndmNoise
	rndmNoise = NaN
	
	Variable counter, num_type1, num_type2, num_type3, num_type4 //used to remove points
	For (counter = Lower_limit_P; counter < (upper_limit_P + 1); counter += 1)

		If ((Tags_ch1[counter] == 1) && (numtype(Amp_wave_ch1[counter]) == 0)) // If event on chan 1 is selected and is not a NaN (you can also put a threshold here)
			
			//Data value
			TEMPO_Deltas[(counter - Lower_limit_P)] = Ratio_calculator(peak_wave_ch1, True_wave_ch2, counter, Lag_to_peak, AP_rise_time, Bsl_width, peak_win_width, random_window, low_lim = lower_lim, up_lim = upper_lim)
			
			//noise value counter
			rndmNoise[(counter - Lower_limit_P)] = Ratio_calculator(peak_wave_ch1, True_wave_ch2, counter, Lag_to_peak, AP_rise_time, Bsl_width, peak_win_width, random_window, randomize = 1, low_lim = lower_lim, up_lim = upper_lim)
			//noise value counter'
			rndmNoise[((counter - Lower_limit_P) + numpnts(TEMPO_Deltas))] = Ratio_calculator(peak_wave_ch1, True_wave_ch2, counter, Lag_to_peak, AP_rise_time, Bsl_width, peak_win_width, random_window, randomize = 1, low_lim = lower_lim, up_lim = upper_lim)
			//noise value counter''
			rndmNoise[((counter - Lower_limit_P) + (2 * numpnts(TEMPO_Deltas)))] = Ratio_calculator(peak_wave_ch1, True_wave_ch2, counter, Lag_to_peak, AP_rise_time, Bsl_width, peak_win_width, random_window, randomize = 1, low_lim = lower_lim, up_lim = upper_lim)
			
			//Check the type of number stored at that step in each wave
			num_type1 = numtype(TEMPO_Deltas[(counter - Lower_limit_P)])
			num_type2 = numtype(rndmNoise[(counter - Lower_limit_P)])
			num_type3 = numtype(rndmNoise[((counter - Lower_limit_P) + numpnts(TEMPO_Deltas))])
			num_type4 = numtype(rndmNoise[((counter - Lower_limit_P) + (2 * numpnts(TEMPO_Deltas)))])
			
			// If one of the numbers was not a normal number (NaN or Inf),  all other numbers at that step are set to NaN to be removed
			If ((num_type1 + num_type2 + num_type3 + num_type4) != 0)
				TEMPO_Deltas[(counter - Lower_limit_P)] = NaN
				rndmNoise[(counter - Lower_limit_P)] = NaN
				rndmNoise[((counter - Lower_limit_P) + numpnts(TEMPO_Deltas))] = NaN
				rndmNoise[((counter - Lower_limit_P) + (2 * numpnts(TEMPO_Deltas)))] = NaN
			EndIf	
			
		EndIf
	
	EndFor
	
	RemoveNaNs(TEMPO_Deltas) ; RemoveNaNs(rndmNoise)//Remove empty lines (when Tags_ch1 == 0), or one value was out of range

End




Function Ratio_calculator(peak_wave_ch1, True_wave_ch2, counter, Lag_to_peak, AP_rise_time, Bsl_width, peak_win_width, random_window, [randomize, low_lim, up_lim])
	Wave peak_wave_ch1, True_wave_ch2
	Variable counter, Lag_to_peak, AP_rise_time, Bsl_width, peak_win_width, random_window, randomize, low_lim, up_lim
	//randomize should be mentionned if TEMPO_peak_position is expected to be random (optionnal)
	Variable the_ratio, TEMPO_peak_position, TEMPO_Bsl_mean, TEMPO_Peak_mean, rndm //TEMPO_peak_position is location of peak nb N on chan2
	
		//__________________________________check if we use the random peak_poistion___________________
		If (ParamIsDefault(randomize))
			TEMPO_peak_position = (peak_wave_ch1[counter] + Lag_to_peak)
			
		Else
			rndm = Abs(enoise(random_window)) // random_window in second will return rndm in seconds
			TEMPO_peak_position = ((peak_wave_ch1[counter] - AP_rise_time) - rndm) //True peak position minus the random nb (peak_win_width is used to prevent averaging the AP artifact)
		
		EndIf
		//___________________________________Check If values are in range_______________
		
		// ___________________________if not the ratio will be NaN_______________________
		// If we are at the end of the sweep (the peak location on ch2 would be out => value would be zero)
		If (((TEMPO_peak_position + (peak_win_width / 2)) > up_lim))
			The_ratio = NaN
			Nvar nb_values_inf_to_range
			nb_values_inf_to_range += 1
		// If the begginning of the baseline is before the begginning of the sweep
		ElseIf ((peak_wave_ch1[counter] - (Bsl_width + AP_rise_time)) < low_lim) // Begining of bsl range
			The_ratio = NaN
			Nvar nb_values_sup_to_range
			nb_values_sup_to_range += 1
		// If the value is in range
		Else // _____________________If we are the ratio will be computed________________
			WaveStats/Q/M=1/R=((peak_wave_ch1[counter] - (Bsl_width + AP_rise_time)), (peak_wave_ch1[counter] - AP_rise_time)) True_wave_ch2
			TEMPO_Bsl_mean = V_avg
			WaveStats/Q/M=1/R=((TEMPO_peak_position - (peak_win_width / 2)), TEMPO_peak_position + (peak_win_width / 2)) True_wave_ch2
			TEMPO_Peak_mean = V_avg
		
			The_ratio =  (TEMPO_Peak_mean - TEMPO_Bsl_mean)
			If (the_ratio == 0) // If the ratio is absolutely zero remove it (as it should not exist statistically, it means we average a null portion of wave)
				the_ratio =NaN
				Nvar nb_values_inf_to_range
				nb_values_inf_to_range += 1
			EndIf
		EndIf
					

	Return the_ratio
		
End

Function MASproc(ctrlName) : ButtonControl
	string ctrlName
	
	string chNb = ctrlName[strlen(ctrlName) - 1]
	// determining the complementary channel
	string otherChNb
	if (cmpstr(chNb, "1") == 0) // CAUTION inverted to invert the target window
		otherChNb = "1"
	elseif (cmpstr(chNb, "2") == 0)
		otherChNb = "2"
	endif


	// ___________________________________________________Creting numeric and string variables____________________________________
	Variable tempStart, tempEnd, Spacan_ampl
	Variable spac_sum, spac_nb, lower_limit_P2, upper_limit_P2
	
	// To check the number of values out of range
	Variable/G nb_values_inf_to_range = 0, nb_values_sup_to_range = 0
	
	Variable/G alpha
	Variable max_noise, numBins
	Variable nb_succes, spikesFreq
	Variable Lower_limit_P, Upper_limit_P	
	
	//________________________________________________Globals________________________________________________________
	
	NVar peakWinWidthG = $("MASPeakWinWidth" + chNb)
	NVar bslWidthG = $("MASBslWidth" + chNb)
	NVar spikesRiseTimeG = $("MASAPRiseTime" + chNb)
	NVar lag2PeakG = $("MASLagToPeak" + chNb)
	NVar histBinSizeG = $("MASTrueHistBinSize" + chNb)
//	NVar noiseHistBinSizeG = $("MASNoiseHistBinSize" + chNb)
//	NVar histMinG = $("MASHistMin" + chNb)
//	NVar histMaxG = $("MASHistMax" + chNb)
	NVAR MASStart = $("MASStart" + chNb)
	NVAR MASEnd = $("MASEnd" + chNb)
	SVAR MASPharmaco = $("MASPharmaco" + chNb)
	SVar fileName = $("filename" + otherChNb)
	print "Target wave is : " + fileName	
	
	tempStart = MASStart; tempEnd = MASEnd
	// ____________________________________Waves references________________________________________
	wave spikesOccurrence = $("occurrence" + chNb)
	wave eventsOccurrence = $("occurrence" + otherChNb)
	wave spikesTags = $("tags" + chNb)
	wave eventsTags = $("tags" + otherChNb)
	wave spikesAmpl = $("realAmp" + chNb)
	duplicate/O $("realAmp" + otherChNb), eventsAmpl
	duplicate/O $fileName, eventsChannel
	
	if ((sum(spikesTags) == 0) || (sum(eventsTags) == 0))
		abort "This function uses tags. Tag events first on both channels."
	endIf
	// __________________________ using local variables to convert the units___________________________
	Variable peakWinWidth, bslWidth, spikesRiseTime, lag2Peak, histBinSize
	Variable histMin, histMax


	//convert values from ms to seconds
	peakWinWidth = peakWinWidthG * 0.001
	bslWidth = bslWidthG * 0.001
	spikesRiseTime = spikesRiseTimeG * 0.001
	lag2Peak = lag2PeakG * 0.001
	
	//Convert values from pA to Ampers
	histBinSize = histBinSizeG * 1e-12

	//______________________________Creates the output waves____________________________________
	Make/O/N=(1, 10) MASMatriX = NaN //Creates the output stats wave with 9 columns
	// The default value is NaN to see where no calculus was performed
	// Setting of columns labels
	SetDimLabel 1, 0, sweepStart, MASMatriX
	SetDimLabel 1, 1, sweepEnd, MASMatriX
	SetDimLabel 1, 2, failure_rate, MASMatriX
	SetDimLabel 1, 3, nb_events, MASMatriX
	SetDimLabel 1, 4, ampl, MASMatriX
	SetDimLabel 1, 5, SpAcAn_ampl, MASMatriX
	SetDimLabel 1, 6, ampl_fail, MASMatriX
	SetDimLabel 1, 7, spike_freq, MASMatriX
	SetDimLabel 1, 8, nb_spikes, MASMatriX
	SetDimLabel 1, 9, AmplFailSD, MASMatriX
	
	MASMatriX [] [0] = tempStart // Filling the two first columns
	MASMatriX [] [1] = tempEnd
	
	
	// __________________removing events with NaN occurrence__________________________

	ProcDeleteNaN(chNb) // This will untag events if they have a NaN occurence time on reference channel
	ProcDeleteNaN(otherChNb) // This will untag events if they have a NaN occurence time on events channel
	
	
	variable counter = 0
	MAS(spikesOccurrence, spikesTags, eventsChannel, spikesAmpl, peakWinWidth, bslWidth, spikesRiseTime, lag2Peak, lower_lim = tempStart, upper_lim = tempEnd, compteur = counter)
	Wave rndmNoise, TEMPO_Deltas // Import the wave generated by the function	
	
	waveStats/Q/M=1 TEMPO_Deltas
	variable minData, maxData, minNoise, maxNoise
	minData = V_Min
	maxData = V_Max
	waveStats/Q/M=1 rndmNoise
	minNoise = V_Min
	maxNoise = V_Max
	
	histMin = min(minData, minNoise) - (3*histBinSize) 
	histMax = max(maxData, maxNoise) + (3*histBinSize)
	
	//________________________set locals values
	numBins = ((histMax - histMin) / histBinSize)
	numBins = ((histMax - histMin) / histBinSize)
	
	Make/O/N=(numBins) dataHist, noiseHist

	//_______________Calculation w/ original SpAcAn method w/ Tags2 sync from Tags1
	Lower_limit_P2 = X_finder(eventsOccurrence, tempStart, supBorder=1) // take first event superior 2 tempStart
	Upper_limit_P2 = X_finder(eventsOccurrence, tempEnd, infBorder=1) // take last event inferior 2 tempEnd
	//Calculation of SpAcAn amplitude (w/  tags2 synchronysed)
	
	eventsAmpl *= eventsTags // Set amplitude of untagged events to 0
	
	spac_sum = Sum(eventsAmpl, Lower_limit_P2, Upper_limit_P2)
	spac_nb = Sum(eventsTags, Lower_limit_P2, Upper_limit_P2)
	Spacan_ampl = (spac_sum / spac_nb)
	
	//____________figures w/ MAS method_______________________________
	If (numpnts(TEMPO_Deltas) > 2) //Minimum number of events required to do an histogram
		//Defines the limits of the histogram 
		Wavestats/Q rndmNoise
		max_noise = V_max

		//Creates the histogram wave and display
		Histogram/B={histMin, histBinSize, numBins} TEMPO_Deltas, dataHist
		Histogram/B={histMin, histBinSize, numBins} rndmNoise, noiseHist
		
		// Define final and unique wave names
		String DataName = ("Data_" + MASPharmaco)
		String NoiseName = ("Noise_" + MASPharmaco)
		Duplicate/O dataHist, $DataName// Save the histogram waves
		Duplicate/O noisehist, $NoiseName
		
		// Graph Window
		DoWindow/K TempHistName
		Display/K=1/N=TempHistName/B=amplitude/R=nb_events $DataName, $NoiseName
//		SetAxis/W=TempHistName amplitude, -500e-12,50e-12
		DoWindow/K $("Hist" + MASPharmaco)
		RenameWindow TempHistName, $("Hist" + MASPharmaco)
		
		// Graph Layout
		ModifyGraph mode=5, hbfill=2 // Curve properties
		ModifyGraph rgb($DataName)=(65535, 0, 0), rgb($NoiseName)=(0, 0, 65535) // Colors
		ModifyGraph freePos(amplitude)={0,nb_events}, freePos(nb_events)={0,kwFraction} // Axis
		Label nb_events "Events (n)"
		Label amplitude "Amplitude (A)"
		ModifyGraph margin=40, axThick=2
		ModifyGraph lblPosMode(amplitude)=1
		//Legend
		SetDrawEnv xcoord= rel,ycoord= rel,textrgb= (65280,0,0), fstyle= 1
		DrawText 0.65,0.2,"Data (n=" + num2str(sum(dataHist)) + ")"
		SetDrawEnv xcoord= rel,ycoord= rel,textrgb= (0,12800,52224), fstyle= 1
		DrawText 0.65,0.3,"Noise"
		
		//______________Calculation of the alpha scaling factor to fit the noise wave to the data wave______
		
		// finds the point in peak_wave_ch1 closest to lower_limit_P (point of the wave where analysis beggins)
		Lower_limit_P = X_finder(spikesOccurrence, tempStart, supBorder=1)
		Upper_limit_P = X_finder(spikesOccurrence, tempEnd, infBorder=1)
		
		alpha = hist_fit(dataHist, noiseHist, 0, max_noise) // We just fit the positive part of the noise curve
		noiseHist *= alpha // Scaling of the noise curve
		Duplicate/O noiseHist, $NoiseName // Overwrites the wave to update display after scaling
		
		alpha *=  ((numpnts(rndmNoise)) / (numpnts(TEMPO_Deltas))) //since we have several noise values per sample
		
		// ******************************************************************
		If (((numpnts(rndmNoise)) / (numpnts(TEMPO_Deltas))) != round((numpnts(rndmNoise)) / (numpnts(TEMPO_Deltas))))
			Abort
		EndIf
		//*****************************************************************
		
		//______________Filling the matriX w/ values____________________________________________
		
		
		nb_succes = round((numpnts(TEMPO_Deltas) - (alpha * numpnts(TEMPO_Deltas)))) //number of successes
		
		WaveStats/Q TEMPO_Deltas
		//if (V_Flag )
			spikesFreq = (Sum(spikesTags, Lower_limit_P, Upper_limit_P) / (tempEnd - tempStart))
			MASMatriX [counter] [2] = alpha // Insert the failure ratio value in the stat matrix
			MASMatriX [counter] [4] = (V_Sum / nb_succes) //amplitude of current sweep without failures
			MASMatriX [counter] [5] = Spacan_ampl // Amplitude w/ SpAcAn tags2 synchronysed from tags1
			MASMatriX [counter] [6] = V_Avg //amplitude of current sweep w/ failures
			MASMatriX [counter] [7] = spikesFreq //freq of AP in current sweep
			MASMatriX [counter] [8] = (Sum(spikesTags, Lower_limit_P, Upper_limit_P)) // Number of actionpotentials in the sweep
			MASMatriX [counter] [9] = V_sdev
			MASMatriX [counter] [3] = V_npnts //number of events
		//endIf

	EndIf
	//_________________CreatingThe whole wave and killing locals______________________
	Print "the noise pnt nb for step " + num2str(counter) + "is : " + num2str(numpnts(rndmNoise))
	Print "the data pnt nb for step " + num2str(counter) + "is : " + num2str(numpnts(TEMPO_Deltas)) 

	Killwaves rndmNoise, TEMPO_Deltas, dataHist, noiseHist
	
	Killvariables Lower_limit_P, Upper_limit_P
	duplicate/O MASMatriX, $("MASMatriX_" + MASPharmaco)
	DoWindow/K tempMatriXWinN
	Edit/N=tempMatriXWinN/k=1 $("MASMatriX_" + MASPharmaco).ld //.ld = show labels
	DoWindow/K $(MASPharmaco + "MatriX")
	RenameWindow TempMatriXWinN, $(MASPharmaco + "MatriX")
	//Check the nb of values that would have been out of range (for curisosity)
	Print "Number of values inferior to range : " + num2str(nb_values_inf_to_range) + "\r Number of values superior to range : " + num2str(nb_values_sup_to_range)
	
	//Kill useless waves
	Killwaves eventsChannel, eventsAmpl, MASMatriX
	KillVariables alpha
End

// Finds the location as a P index of the value in the Target_wave (X_value is specified because Tagrget_wave shall be monotonous)
//If X_value equidistant from two values, the lowest value is taken (except if U use the options
Function X_finder(Target_wave, X_value, [SupBorder, InfBorder]) //Put =1 if you want to use the optionnal parameter, SupBorder means take the point that is >= to X_value (1/X for InfBorder)
	Wave Target_wave
	Variable X_value
	Variable Supborder, InfBorder
	
	Variable P_value
	
	Duplicate/O Target_wave, delta_index_W
		
	delta_index_W -= X_value // fills the wave with the difference between each point and the value
	delta_index_W = Abs(delta_index_W) // trick to avoid negative values to be taken into account
	
	WaveStats/Q delta_index_W
	P_value = X2pnt(delta_index_W, V_minLoc) //returns the location in point of the minimum value in the wave
	Killwaves delta_index_W
	
	If (!ParamIsDefault(SupBorder)) // If parameter was used
		If (Target_wave[P_value] < X_value) // If value inferior increase by one
			P_value += 1
		EndIf
		For (; ((numtype(Target_wave[P_value]) == 2) && (P_value < numpnts(Target_wave))); ) // If value is NaN switch to next
			P_value += 1
		EndFor
	ElseIf (!(ParamIsDefault(InfBorder))) // If parameter was used
		If (Target_wave[P_value] > X_value) // If value superior decrease by one
			P_value -= 1
		EndIf
		For (; ((numtype(Target_wave[P_value]) == 2) && (P_value >= 0)); ) // If value is NaN switch to previous
			P_value -= 1
		EndFor
	EndIf
	
	Return P_value
End


Function hist_fit(wave_to_fit, wave_to_scale, begin, ending) //Take care begin and end are in X scaling (use pnt2X if you want to use point indexing)
	Wave wave_to_fit, wave_to_scale
	Variable begin, ending
	
	If (numpnts(wave_to_fit) != numpnts(wave_to_scale))
		Abort "Can't compute the Least Square Fit with two different distribution sizes"
	EndIf
	
	//The local variable for output
	variable my_coefficient
	
	Make/O/N=((ending - begin) + 1) products, squares
	
	products = wave_to_fit(begin + x) * wave_to_scale(begin + x)
	squares = wave_to_scale(begin + x) * wave_to_scale(begin + x)
	
	my_coefficient = (sum(products, 0, numpnts(products) - 1) / sum(squares, 0, numpnts(squares) - 1))
	Killwaves products, squares
	return my_coefficient
	//The least square difference equation is : sum ((wave_to_fit - wave_to_scale)) that we have to minimize
	// <=> wave_to_fit - 2*wave_to_fit_*wave_to_scale + wave_to_scale
	// With an @ scaling coefficient and the wave_to_fit as a constant, we have the relation :
	//cste - 2*alpha*wave_to_fit*wave_to_scale + alpha*wave_to_scale
	// Relation of the form C + Bx + Ax that will be minimum for  -(B + sqrt(delta)) / 2*A
	//We can also consider the null derivative : 2*wave_to_scale*alpha - 2*wave_to_fit*wave_to_scale = 0
	//with products and squares : alpha = 2*products / 2*squares = my_coefficient  

End