#pragma rtGlobals=1		// Use modern global access method.




//***********************************A band pass filter*******************************
Function AmplitudeRangeFilter(ChannelNb, LowerAmpl, UpperAmpl)
	Variable ChannelNb, LowerAmpl, UpperAmpl
	
	
	
	// Locals
	Variable TrueLowerAmpl, TrueUpperAmpl, LowerP, UpperP // The point indexing of the selected amplitudes
	
	// If lower and upper misunderstood according to the fact that amplitudes are negative
	TrueLowerAmpl = min(LowerAmpl, UpperAmpl)
	TrueUpperAmpl = max(LowerAmpl, UpperAmpl)
	LowerAmpl = TrueLowerAmpl
	UpperAmpl = TrueUpperAmpl
	
	
	DoAlert 1, "Are U sure your tags are backed up ?"
	
	If (V_Flag == 2)
		Abort "Back up your tags !"
	EndIf
	
	
	Duplicate/O $("RealAmp" + num2Str(ChannelNb)), AmplWSorted
	Duplicate/O $("Tags" + num2str(ChannelNb)), TagsWSorted, PWave
	PWave = p
	
	Sort/R AmplWSorted, AmplWSorted, TagsWSorted, PWave // Fine since at that step AmplWSorted is not supposed to be sorted ??!!??
	// We use /R because amplitudes are negative
	// Find the point index of the selected amplitude values in the sorted amplitude wave
	LowerP = X_Finder(AmplWSorted, LowerAmpl, SupBorder=1) // Make sure its upperLim
	UpperP = X_Finder(AmplWSorted, UpperAmpl, InfBorder=1) // Make sure its LowerLim
	
	// Search for these indexes in the sorted tags wave
	// To keep original untagging we are going to the non selected segments instead of tagging the selected one
	// Lower and upper seem inverted still because of negative amplitudes
	TagsWSorted[0, UpperP]=0 // Untag the begining
	TagsWSorted[LowerP, numpnts(TagsWSorted)]=0 // Untag the end
	
	// reorder TagsW
	Sort PWave, TagsWSorted
	Duplicate/O TagsWSorted, $("Tags" + num2str(ChannelNb)) // Put the result in the original tagsW
	KillWaves AmplWSorted, TagsWSorted, PWave
End

Function Go2NextAmplRange(RangeAmpl)
	Variable RangeAmpl
	NVar BandPassLow, BandPassHigh, BandPassStep

	BandPassLow -= RangeAmpl // Because amplitudes are negative
	Print BandPassLow  //Just to Check

	BandPassHigh -= RangeAmpl // Because amplitudes are negative
	Print BandPassHigh //Just to Check
	
	BandPassStep += 1

	Return BandPassHigh
End


Function AmplRangeSwitch(StartAmpl, EndAmpl[, AmplRangeNb, RangeAmpl]) // StartAmpl is the minimum amplitude (in abs therefore the less negative)
	Variable StartAmpl, EndAmpl, AmplRangeNb, RangeAmpl
	
	//Convert pA to A
	StartAmpl *= 1e-12
	EndAmpl *= 1e-12
	RangeAmpl *= 1e-12
	
	
	// Options
	If (ParamIsdefault (RangeAmpl))
		RangeAmpl = (Abs(EndAmpl) - Abs(StartAmpl)) / AmplRangeNb
	ElseIf (ParamIsDefault(AmplRangeNb))
		AmplRangeNb = (Abs(EndAmpl) - Abs(StartAmpl)) /RangeAmpl
	Else
		Abort "Specify either of optionnal parameters"
	EndIf
	
	// Globals
	Variable/G BandPassLow, BandPassHigh, BandPassStep
	If (BandPassLow == 0) // For First Run
		KinMatriXCreate(AmplrangeNb, "KineticsMatriX")
		BandPassLow = StartAmpl
		Print BandPassLow
		BandPassHigh = StartAmpl - RangeAmpl	// Minus because of negative amplitudes
		Print BandPassHigh

	ElseIf (Go2NextAmplRange(RangeAmpl) < EndAmpl) // Inferioir because negative amplitudes
		BandPassLow = 0 // For next use of the function
		BandPassHigh = 0
		BandPassStep = 0
		Abort "UpperLimit reached"
	EndIf

	
End


Function KinMatriXCreate(NbRows, MatriXName)
	Variable NbRows
	String MatriXName
	
	
	// Create the output waves
	Make/O/N=(NbRows, 9) TempMat=NaN
	Make/O/T/N=(NbRows) AvgEventsN_TW=""
	// Setting of MatriX columns labels
	SetDimLabel 1, 0, StartFit, TempMat
	SetDimLabel 1, 1, EndFit, TempMat
	SetDimLabel 1, 2, Offset, TempMat
	SetDimLabel 1, 3, NbEvents, TempMat
	SetDimLabel 1, 4, TauFast, TempMat
	SetDimLabel 1, 5, PercentFast, TempMat
	SetDimLabel 1, 6, TauSlow, TempMat
	SetDimLabel 1, 7, PercentSlow, TempMat
	SetDimLabel 1, 8, KI, TempMat
	
	
	Duplicate/O TempMat, $MatriXName ; KillWaves TempMat
	
End

Function KinMatriXFill(ChannelNb)
	Variable ChannelNb
	
	// Globals
//	If (!(WaveExists(KineticsMatriX)) || !(WaveExists(AvgEventsN_TW)))
//		KinMatriXCreate(
//	EndIf
	Wave KineticsMatriX, FitParams = $("FitParametersFitDblExp" + num2str(ChannelNb)), TagsW = $("Tags" + num2str(ChannelNb))
	Wave/T AvgEventsN_TW
	NVar BandPassLow, BandPassHigh, BandPassStep, StartFit = $("StartFit" + num2str(ChannelNb)), EndFit = $("EndFit" + num2str(ChannelNb))
	
	//locals
	Variable Offset, TauFast, PercentFast, TauSlow, PercentSlow, KI, NbEvents
	String AvgEvntDestN = "AvgEvCh" + num2str(ChannelNb) + "_" + SciNum2Str(BandPassLow) + "To" + SciNum2Str(BandPassHigh)
	String BaseN = "Ch" + num2str(ChannelNb) + "_" + SciNum2Str(BandPassLow) + "To" + SciNum2Str(BandPassHigh)
	
	Duplicate/O $("AverageEvent" + num2str(ChannelNb)), $(AvgEvntDestN)
	Duplicate/O $("Fit_AverageEvent" + num2str(ChannelNb)), $("Fit" + AvgEvntDestN)
	Duplicate/O TagsW, $("Tags" + BaseN)
	Display/k=1 $(AvgEvntDestN)
	ModifyGraph rgb($(AvgEvntDestN))=(0,15872,65280)
	AppendToGraph $("Fit" + AvgEvntDestN)
	
	AvgEventsN_TW[BandPassStep] = AvgEvntDestN
	
	TauFast = FitParams[0]
	PercentFast = FitParams[2]
	TauSlow = FitParams[1]
	PercentSlow = FitParams[3]
	KI = (PercentFast/100)*TauFast + (PercentSlow/100)*TauSlow
	Offset = FitParams[8]
	NbEvents = Sum(TagsW)
	KineticsMatriX[BandPassStep][%Offset] = Offset
	KineticsMatriX[BandPassStep][%TauFast] = TauFast
	KineticsMatriX[BandPassStep][%PercentFast] = PercentFast
	KineticsMatriX[BandPassStep][%TauSlow] = TauSlow
	KineticsMatriX[BandPassStep][%PercentSlow] = PercentSlow
	KineticsMatriX[BandPassStep][%KI] = KI
	KineticsMatriX[BandPassStep][%StartFit] = StartFit
	KineticsMatriX[BandPassStep][%EndFit] = EndFit
	KineticsMatriX[BandPassStep][%nbEvents] = NbEvents

End


Function/S SciNum2Str(num)
	Variable num
	
	String Output = num2str(num)
	
	Output = ReplaceString("-", Output, "M")
	Output = ReplaceString("+", Output, "P")
//	Output = ReplaceString("e", Output, "E")
	Output = ReplaceString(".", Output, "_")
	Return Output
End
